<?php
/**
 * @filename Tokenizer.php
 * @author Jerry Yan <792602257@qq.com>
 * @date 2020/12/17 15:22
 */


namespace JerryYan\DSL\Tokenizer;


use JerryYan\DSL\Lexer\LexerInterface;
use JerryYan\DSL\Reader\ReaderInterface;
use JerryYan\DSL\Token\TokenInterface;

class Tokenizer extends TokenizerInterface
{
    /** @var array<string, class-string<TokenInterface>> Token类型及映射类 */
    protected $tokenMapping = [];
    /** @var array<string, class-string<TokenInterface>> Token别名映射 */
    protected $tokenNameMapping = [];
    /** @var array<string, class-string<TokenInterface>> Token别名映射 */
    protected $regexNameMapping = [];
    /** @var LexerInterface 语法器 */
    protected $lexer;

    public function __construct(LexerInterface $lexer)
    {
        $this->lexer = $lexer;
        /**
         * @var string $key
         * @var TokenInterface $token
         */
        foreach ($lexer->getTokenTypes() as $key=> $token) {
            $this->tokenMapping[$key] = $token;
            foreach ($token::$alias as $name) {
                $this->tokenNameMapping[$name] = $key;
            }
            foreach ($token::$regexAlias as $name) {
                $this->regexNameMapping[$name] = $key;
            }
        }
    }

    /**
     * @inheritDoc
     */
    function tokenize(ReaderInterface $reader): ?TokenInterface
    {
        $reader->resetCursor();
        /** @var TokenInterface $lastToken */
        $lastToken = NULL;
        do
        {
            $currentTokenName = $reader->getCurrentToken();
            $currentToken = $this->getTokenByName($currentTokenName);
            $currentToken->setPrevToken($lastToken);
            if ($lastToken !== NULL) $lastToken->setNextToken($currentToken);
            $lastToken = $currentToken;
        }while($reader->moveToNextToken());
        return $lastToken->getFirstToken();
    }

    /**
     * 根据文字获取Token
     * @param string $name
     * @return TokenInterface
     * @author Jerry Yan <792602257@qq.com>
     * @date 2021/1/26 16:41
     */
    protected function getTokenByName(string $name): TokenInterface
    {
        $originalName = $name;
        if (isset($this->tokenNameMapping[$name])) {
            $name = $this->tokenNameMapping[$name];
        } else {
            foreach ($this->regexNameMapping as $regex => $newName) {
                if (preg_match($regex, $name) === 1) {
                    $name = $newName; break;
                }
            }
        }
        if (!isset($this->tokenMapping[$name])) {
            $undefinedType = $this->lexer->getUndefinedTokenType();
            return new $undefinedType($originalName);
        } else {
            return new $this->tokenMapping[$name]($originalName);
        }
    }
}
